@@ -7,9 +7,9 @@
 
 /* -------------------------------------------------------------------------- */
 
-struct game_ai_s const *game_ai = &game_ai_classic;
+const struct game_ai_s *game_ai = &game_ai_classic;
 
-const struct game_ai_s const *game_ais[GAME_AI_NUM] = {
+const struct game_ai_s *const game_ais[GAME_AI_NUM] = {
     &game_ai_classic,
     &game_ai_classicplus
 };
