@@ -416,10 +416,10 @@ static void game_battle_missile_move(struct battle_s *bt, int missile_i, int tar
             for (int i = 0; i < bt->num_rocks; ++i) {
                 struct battle_rock_s *r = &(bt->rock[i]);
                 if (1
-                  && ((r->sx * 32 + 2) >= mx0)
-                  && ((r->sx * 32 + 30) < mx0)
-                  && ((r->sy * 24 + 2) >= my0)
-                  && ((r->sy * 24 + 22) < my0)
+                  && ((r->sx * 32 + 2) <= mx0)
+                  && ((r->sx * 32 + 30) > mx0)
+                  && ((r->sy * 24 + 2) <= my0)
+                  && ((r->sy * 24 + 22) > my0)
                 ) {
                     collisions += game_battle_missile_rock_collide(bt, m);
                 }
@@ -971,7 +971,9 @@ static bool game_battle_special(struct battle_s *bt, int attacker_i, int target_
                     break;
             }
             if (bd->hp1 <= bd->hploss) {
-                --bd->num;
+                if (bd->num > 0) {
+                    --bd->num;
+                }
                 bd->hploss = 0;
             }
             if ((bd->num <= 0) || (bd->hp1 <= 0)) {
@@ -1330,9 +1332,9 @@ static void game_battle_with_human_do_sub3(struct battle_s *bt)
                         b->retreat = 1;
                     } else {
                         game_battle_with_human_do_turn_ai(bt);
+                        flag_turn_done = true;
+                        game_battle_item_done(bt);
                     }
-                    flag_turn_done = true;
-                    game_battle_item_done(bt);
                 } else {
                     /*4ece2*/
                     act = ui_battle_turn(bt);
