@@ -23,12 +23,36 @@ bool game_cheat_galaxy(struct game_s *g, player_id_t pi)
     return true;
 }
 
+bool game_cheat_elections(struct game_s *g, player_id_t pi)
+{
+    g->gaux->flag_cheat_elections = !g->gaux->flag_cheat_elections;
+    return true;
+}
+
 bool game_cheat_events(struct game_s *g, player_id_t pi)
 {
     g->gaux->flag_cheat_events = !g->gaux->flag_cheat_events;
     return true;
 }
 
+bool game_cheat_spy_hint(struct game_s *g, player_id_t pi)
+{
+    g->gaux->flag_cheat_spy_hint = !g->gaux->flag_cheat_spy_hint;
+    return true;
+}
+
+bool game_cheat_stars(struct game_s *g, player_id_t pi)
+{
+    g->gaux->flag_cheat_stars = !g->gaux->flag_cheat_stars;
+    return true;
+}
+
+bool game_cheat_tech_hint(struct game_s *g, player_id_t pi)
+{
+    g->gaux->flag_cheat_tech_hint = !g->gaux->flag_cheat_tech_hint;
+    return true;
+}
+
 bool game_cheat_moola(struct game_s *g, player_id_t pi)
 {
     empiretechorbit_t *e = &(g->eto[pi]);
