@@ -12,6 +12,7 @@
 #include "game_shiptech.h"
 #include "game_str.h"
 #include "game_tech.h"
+#include "lib.h"
 #include "log.h"
 #include "rnd.h"
 #include "types.h"
@@ -72,7 +73,7 @@ static void game_get_random_shipnames(struct game_s *g, player_id_t player, char
         if ((namei >= 0) && (namei < SHIP_NAME_NUM)) {
             BOOLVEC_SET0(name_unused, namei);
         }
-        strcpy(shipnames[i], names[namei]); /* TODO weird that the strcpy is not inside the previous if */
+        lib_strcpy(shipnames[i], names[namei], SHIP_NAME_LEN + 1); /* TODO weird that the strcpy is not inside the previous if */
     }
 }
 
@@ -181,6 +182,47 @@ static void game_design_prepare_do(struct game_s *g, struct game_design_s *gd, p
     memcpy(gd->percent, e->tech.percent, sizeof(gd->percent));
 }
 
+static weapon_group_t game_design_get_weapon_group(const struct game_s *g, const struct shiptech_weap_s *w)
+{
+    tech_field_t field;
+    tech_group_t group;
+    if (w->tech_i > 50) {
+        return WEAPON_GROUP_SPECIAL;
+    }
+    field = w->is_bio ? TECH_FIELD_PLANETOLOGY : TECH_FIELD_WEAPON;
+    group = game_tech_get_group(g->gaux, field, w->tech_i);
+    if (group == TECH_GROUP_SINGULAR) {
+        if (w->is_bio) {
+            return WEAPON_GROUP_BIOLOGICAL;
+        }
+        if (w->is_bomb) {
+            return WEAPON_GROUP_BOMBS;
+        }
+        if (w->halveshield) {
+            return WEAPON_GROUP_KINETIC;
+        }
+        if (w->damagemin != w->damagemax) {
+            return WEAPON_GROUP_BEAMS;
+        }
+        return WEAPON_GROUP_MISSILES;
+    }
+    switch (group) {
+        case TECH_GROUP_BEAMS:
+            return WEAPON_GROUP_BEAMS;
+        case TECH_GROUP_PENETRATING_BEAMS:
+            return WEAPON_GROUP_KINETIC;
+        case TECH_GROUP_BIO_WEAPONS:
+            return WEAPON_GROUP_BIOLOGICAL;
+        case TECH_GROUP_BOMBS:
+            return WEAPON_GROUP_BOMBS;
+        case TECH_GROUP_ROCKETS:
+            return WEAPON_GROUP_MISSILES;
+        default:
+            break;
+    }
+    return WEAPON_GROUP_NUM;
+}
+
 /* -------------------------------------------------------------------------- */
 
 void game_design_prepare(struct game_s *g, struct game_design_s *gd, player_id_t player, shipdesign_t *sd)
@@ -216,7 +258,7 @@ void game_design_prepare_ai(struct game_s *g, struct game_design_s *gd, player_i
     game_design_prepare_do(g, gd, player, 0);
     sd->hull = hull;
     sd->look = look;
-    strcpy(sd->name, gd->names[hull]);
+    lib_strcpy(sd->name, gd->names[hull], SHIP_NAME_LEN);
     game_design_update_engines(sd);
     sd->space = game_design_calc_space(gd);
     sd->cost = game_design_calc_cost(gd);
@@ -424,11 +466,11 @@ void game_design_clear(struct game_design_s *gd)
     char name[SHIP_NAME_LEN + 1];
     uint8_t look = sd->look;
     ship_hull_t hull = sd->hull;
-    strcpy(name, sd->name);
+    lib_strcpy(name, sd->name, sizeof(name));
     memset(sd, 0, sizeof(shipdesign_t));
     sd->look = look;
     sd->hull = hull;
-    strcpy(sd->name, name);
+    lib_strcpy(sd->name, name, SHIP_NAME_LEN);
     game_design_update_engines(sd);
     sd->space = game_design_calc_space(gd);
     sd->cost = game_design_calc_cost(gd);
@@ -591,7 +633,7 @@ int game_design_build_tbl_fit_man(struct game_s *g, struct game_design_s *gd, in
     return sd->engine;
 }
 
-int game_design_build_tbl_fit_weapon(struct game_s *g, struct game_design_s *gd, int8_t *buf, int wslot)
+int game_design_build_tbl_fit_weapon(struct game_s *g, struct game_design_s *gd, int8_t *buf, int wslot, weapon_group_t wgroup)
 {
     shipdesign_t *sd = &(gd->sd);
     weapon_t actwpnt = sd->wpnt[wslot];
@@ -600,8 +642,12 @@ int game_design_build_tbl_fit_weapon(struct game_s *g, struct game_design_s *gd,
     buf[0] = 1/*HAVE*/;
     for (int i = 1; i < WEAPON_NUM; ++i) {
         tech_field_t fi;
+        weapon_group_t group;
+        bool match;
         fi = tbl_shiptech_weap[i].is_bio ? TECH_FIELD_PLANETOLOGY : TECH_FIELD_WEAPON;
-        if (game_tech_player_has_tech(g, fi, tbl_shiptech_weap[i].tech_i, gd->player_i)) {
+        group = game_design_get_weapon_group(g, &(tbl_shiptech_weap[i]));
+        match = (wgroup == WEAPON_GROUP_ALL) || (wgroup == group);
+        if (game_tech_player_has_tech(g, fi, tbl_shiptech_weap[i].tech_i, gd->player_i) && match) {
             uint8_t n;
             sd->wpnt[wslot] = i;
             /* TODO use binary search */
