@@ -27,6 +27,8 @@
 
 /* -------------------------------------------------------------------------- */
 
+bool game_opt_skip_random_news = false;
+
 /* -------------------------------------------------------------------------- */
 
 static player_id_t game_event_new_get_victim(struct game_s *g)
@@ -222,7 +224,7 @@ void game_event_new(struct game_s *g)
         type = rnd_1_n(GAME_EVENT_NUM - 1, &g->seed);
         if (0
           || BOOLVEC_IS1(g->evn.done, type)
-          || ((type == GAME_EVENT_REBELLION) && (IS_HUMAN(g, player) || (g->evn.home[player] == planet)))
+          || ((type == GAME_EVENT_REBELLION) && (IS_AI(g, player) || (g->evn.home[player] == planet)))
           || ((type == GAME_EVENT_QUAKE) && (p->pop < 50))
           || (((type == GAME_EVENT_CRYSTAL) || (type == GAME_EVENT_AMOEBA)) && (g->year < 200))
           || (((type == GAME_EVENT_CRYSTAL) || (type == GAME_EVENT_AMOEBA)) && (g->evn.crystal.exists || g->evn.amoeba.exists))
@@ -231,6 +233,11 @@ void game_event_new(struct game_s *g)
           || ((type == GAME_EVENT_COMET) && (g->year < 100))
           || ((type == GAME_EVENT_PIRATES) && (game_event_new_get_trader(g, player) == PLAYER_NONE))
           || (((type == GAME_EVENT_ACCIDENT) || (type == GAME_EVENT_PLAGUE)) && (e->race == RACE_SILICOID))
+          /* The OSG claims that the industrial accident event only happens on
+           * planets with at least 30 factories. This is not true for MOO 1.3,
+           * but we optionally check for it if accident_chk_factories is set in
+           * a pbx file. */
+          || ((type == GAME_EVENT_ACCIDENT) && game_num_accident_chk_factories && (p->factories < 30))
           || ((type == GAME_EVENT_ENVIRO) && (p->type < PLANET_TYPE_MINIMAL))
           || ((type == GAME_EVENT_ENVIRO) && ((p->growth >= PLANET_GROWTH_FERTILE) || (p->special > PLANET_SPECIAL_NORMAL)))
           || ((type == GAME_EVENT_RICH) && (p->special > PLANET_SPECIAL_NORMAL))
@@ -912,23 +919,14 @@ bool game_event_run(struct game_s *g, struct game_end_s *ge)
         planet_t *p = &(g->planet[pli]);
         player_id_t player = p->owner;
         empiretechorbit_t *e = 0;
-        int v, terraf = 0;
+        int terraf = 0;
         if (player != PLAYER_NONE) {
             e = &(g->eto[player]);
             terraf = e->have_terraform_n;
         }
         ns.planet_i = pli;
         ns.race = (player != PLAYER_NONE) ? e->race : RACE_HUMAN; /* WASBUG MOO1 does not check for none, taking race from eto[-1] */
-        p->growth = PLANET_GROWTH_FERTILE;
-        v = (p->max_pop1 / 20) * 5;
-        if ((p->max_pop1 / 20) % 5) {
-            v += 5;
-        }
-        SETMAX(v, 5);
-        p->max_pop2 = p->max_pop1 + v;
-        p->max_pop3 += v;
-        SETMIN(p->max_pop3, p->max_pop2 + terraf);
-        SETMIN(p->max_pop3, game_num_max_pop);
+        game_turn_soil_enrich(p, terraf, 0);
         g->evn.have_enviro = false;
         ns.type = GAME_NEWS_ENVIRO;
         ns.subtype = IS_HUMAN(g, player) ? 0 : 4;
@@ -1098,6 +1096,7 @@ bool game_event_run(struct game_s *g, struct game_end_s *ge)
             if (any_in_range) {
                 ui_news(g, &ns);
                 any_news = true;
+                /* WASBUG If multiple coups happen in one turn, MOO1 reports only one. */
             }
             /* TODO What about multiplayer? Set overthrown player to AI? */
         }
@@ -1159,6 +1158,7 @@ bool game_event_run(struct game_s *g, struct game_end_s *ge)
     }
     /*114fb*/
     if (1
+      && (!game_opt_skip_random_news)
       && (!any_news)
       && (!rnd_0_nm1(40, &g->seed))
       && (g->year > 25)
