@@ -12,6 +12,7 @@
 #include "game_shiptech.h"
 #include "game_str.h"
 #include "game_tech.h"
+#include "lib.h"
 #include "log.h"
 #include "rnd.h"
 #include "types.h"
@@ -112,17 +113,27 @@ void game_update_production(struct game_s *g)
         player_id_t owner = p->owner;
         if (owner != PLAYER_NONE) {
             empiretechorbit_t *e = &(g->eto[owner]);
+            int pop = p->pop;
             int v;
-            {
+            if (game_num_leaving_trans_fix) {
+                int rebels = p->rebels;
+                if (p->trans_num) {           /* see game_send_transport() */
+                    pop -= p->trans_num;
+                    SUBSAT0(rebels, p->trans_num / 2 + 1);
+                    SETMAX(pop, 1);
+                }
+                v = (pop - rebels) * game_planet_get_pop_oper_fact(g, p);
+                SETMAX(v, 0);
+            } else {
                 int popx = p->pop - p->rebels;
                 SETMAX(popx, 0);
-                v = popx * e->colonist_oper_factories;
+                v = popx * game_planet_get_pop_oper_fact(g, p);
             }
             {
                 uint16_t factories = p->factories;
                 int extra;
                 SETMIN(factories, v);
-                extra = (p->pop * (e->tech.percent[TECH_FIELD_PLANETOLOGY] * 3 + 50)) / 100;
+                extra = (pop * (e->tech.percent[TECH_FIELD_PLANETOLOGY] * 3 + 50)) / 100;
                 if (e->race == RACE_KLACKON) {
                     extra <<= 1;
                 }
@@ -200,6 +211,8 @@ void game_update_production(struct game_s *g)
             if (p->owner == pi) {
                 int v;
                 v = (p->prod_after_maint * e->percent_prod_total_to_actual) / 100 - p->trans_num;
+                /* BUG: deducting transport cost takes no effect for actual production,
+                 * which happens after departure, but leads to wrong display for player. */
                 p->prod_after_maint = (v > 0) ? v : 0;
             }
         }
@@ -234,54 +247,42 @@ void game_update_eco_on_waste(struct game_s *g, int player_i, bool force_adjust)
     for (int i = 0; i < g->galaxy_stars; ++i) {
         planet_t *p = &(g->planet[i]);
         if (p->owner == player_i) {
-            uint16_t v, fact, waste, prod;
+            uint16_t v;
             int16_t left;
-            fact = p->factories;
-            v = e->colonist_oper_factories * p->pop;
-            SETMIN(fact, v);
-            waste = (fact * e->ind_waste_scale) / 10;
-            waste = (waste + p->waste) / e->have_eco_restoration_n;
-            prod = p->prod_after_maint;
-            if (!prod) {
-                prod = 1000;
-            }
-            v = (waste * 100 + prod - 1) / prod;
-            SETRANGE(v, 0, 100);
-            if ((p->slider[PLANET_SLIDER_ECO] < v) || force_adjust) {
+            bool check_locks = !force_adjust && game_num_slider_respects_locks && IS_HUMAN(g, p->owner);
+            v = game_planet_get_waste_percent(NULL, g, p, false);
+            if (check_locks && p->slider_lock[PLANET_SLIDER_ECO]) {
+            } else if (check_locks && p->slider_lock[PLANET_SLIDER_SHIP] && p->slider_lock[PLANET_SLIDER_DEF] && p->slider_lock[PLANET_SLIDER_IND] && p->slider_lock[PLANET_SLIDER_TECH]) {
+            } else if ((p->slider[PLANET_SLIDER_ECO] < v) || force_adjust) {
+                int16_t eco_diff = v - p->slider[PLANET_SLIDER_ECO];
+                int16_t sum = 100;
                 if (game_num_waste_adjust_fix) {
-                    int sum;
-                    sum = p->slider[PLANET_SLIDER_SHIP] + p->slider[PLANET_SLIDER_DEF] + p->slider[PLANET_SLIDER_IND] + p->slider[PLANET_SLIDER_TECH];
-                    left = sum + p->slider[PLANET_SLIDER_ECO] - v;
-                    SETMAX(left, 0);
-                    if (sum > 0) {
-                        int16_t left2, maxt;
-                        planet_slider_i_t maxi = 0;
-                        maxt = -1;
-                        left2 = left;
-                        for (planet_slider_i_t si = PLANET_SLIDER_SHIP; si <= PLANET_SLIDER_TECH; ++si) {
-                            int16_t t;
-                            if (si == PLANET_SLIDER_ECO) {
-                                continue;
+                    sum = p->slider[PLANET_SLIDER_SHIP] + p->slider[PLANET_SLIDER_DEF] + p->slider[PLANET_SLIDER_IND];
+                }
+                if (check_locks) {
+                    int16_t sum_locks = 0;
+                    for (int si = 0; si < PLANET_SLIDER_NUM; ++si){
+                        if (p->slider_lock[si]) {
+                            sum_locks += p->slider[si];
+                            if (si != PLANET_SLIDER_TECH) {
+                                sum -= p->slider[si];
                             }
-                            p->slider[si] = t = (p->slider[si] * left) / sum;
-                            left2 -= t;
-                            if (maxt < t) {
-                                maxt = t;
-                                maxi = si;
-                            }
-                        }
-                        if (left2 > 0) {
-                            p->slider[maxi] += left2;
                         }
                     }
-                } else {
-                    /* WASBUG this can result in the sum of sliders exceeding 100 */
-                    left = 100 - (v - p->slider[PLANET_SLIDER_ECO]);
-                    p->slider[PLANET_SLIDER_SHIP] = (p->slider[PLANET_SLIDER_SHIP] * left) / 100;
-                    p->slider[PLANET_SLIDER_DEF] = (p->slider[PLANET_SLIDER_DEF] * left) / 100;
-                    p->slider[PLANET_SLIDER_IND] = (p->slider[PLANET_SLIDER_IND] * left) / 100;
+                    SETMIN(v, 100 - sum_locks);
                 }
                 p->slider[PLANET_SLIDER_ECO] = v;
+                if (sum > 0) {
+                    if (!check_locks || !p->slider_lock[PLANET_SLIDER_SHIP]) {
+                        p->slider[PLANET_SLIDER_SHIP] -= (p->slider[PLANET_SLIDER_SHIP] * eco_diff) / sum;
+                    }
+                    if (!check_locks || !p->slider_lock[PLANET_SLIDER_DEF]) {
+                        p->slider[PLANET_SLIDER_DEF] -= (p->slider[PLANET_SLIDER_DEF] * eco_diff) / sum;
+                    }
+                    if (!check_locks || !p->slider_lock[PLANET_SLIDER_IND]) {
+                        p->slider[PLANET_SLIDER_IND] -= (p->slider[PLANET_SLIDER_IND] * eco_diff) / sum;
+                    }
+                }
             }
             SETMAX(p->slider[PLANET_SLIDER_SHIP], 0);
             SETMAX(p->slider[PLANET_SLIDER_DEF], 0);
@@ -293,7 +294,31 @@ void game_update_eco_on_waste(struct game_s *g, int player_i, bool force_adjust)
             left -= p->slider[PLANET_SLIDER_IND];
             left -= p->slider[PLANET_SLIDER_ECO];
             SETMAX(left, 0);
-            p->slider[PLANET_SLIDER_TECH] = left;
+            if (!check_locks || !p->slider_lock[PLANET_SLIDER_TECH]) {
+                p->slider[PLANET_SLIDER_TECH] = left;
+            }
+        }
+    }
+}
+
+void game_update_seen_by_orbit(struct game_s *g, player_id_t pi)
+{
+    empiretechorbit_t *e = &(g->eto[pi]);
+    for (int i = 0; i < g->galaxy_stars; ++i) {
+        planet_t *p = &(g->planet[i]);
+        bool in_orbit;
+        in_orbit = false;
+        for (int j = 0; j < e->shipdesigns_num; ++j) {
+            if (e->orbit[i].ships[j] != 0) {
+                in_orbit = true;
+                break;
+            }
+        }
+        if ((p->owner == pi) || in_orbit) {
+            g->seen[pi][i].owner = p->owner;
+            g->seen[pi][i].pop = p->pop;
+            g->seen[pi][i].bases = p->missile_bases;
+            g->seen[pi][i].factories = p->factories;
         }
     }
 }
@@ -305,7 +330,7 @@ void game_update_within_range(struct game_s *g)
         uint8_t tblplanet_num;
         empiretechorbit_t *e = &(g->eto[pi]);
         bool tbl_alliance[PLAYER_NUM];
-        uint8_t frange, frangep3, srange;
+        uint8_t frange, frangep3, srange, srange2;
         for (int i = 0; i < PLAYER_NUM; ++i) {
             tbl_alliance[i] = (e->treaty[i] == TREATY_ALLIANCE);
         }
@@ -313,6 +338,13 @@ void game_update_within_range(struct game_s *g)
         frange = e->fuel_range;
         frangep3 = frange + 3;
         srange = e->scanner_range;
+        switch (srange) {
+            case 3: srange2 = 0; break;
+            case 5: srange2 = 10; break;
+            case 7: srange2 = 20; break;
+            case 9: srange2 = 30; break;
+            default: break;
+        }
         tblplanet_num = 0;
         for (int i = 0; i < g->galaxy_stars; ++i) {
             planet_t *p = &(g->planet[i]);
@@ -346,6 +378,27 @@ void game_update_within_range(struct game_s *g)
                     p->within_frange[pi] = 0;
                 }
                 BOOLVEC_SET(p->within_srange, pi, (mindist2 <= srange));
+                if (BOOLVEC_IS0(p->within_srange, pi) && ((srange2 > 0) || game_num_ship_scanner_fix)) {
+                    mindist1 = 10000;
+                    for (int j = 0; (j < g->enroute_num) && (mindist1 > srange2); ++j) {
+                        if (g->enroute[j].owner == pi) {
+                            dist = util_math_dist_fast(g->enroute[j].x, g->enroute[j].y, p->x, p->y);
+                            if (game_num_ship_scanner_fix) {
+                                if (dist < mindist1) {
+                                    mindist1 = dist;
+                                }
+                            } else {
+                                dist = (dist + 9) / 10;
+                                if (dist < mindist1) {
+                                    dist = mindist1;
+                                }
+                            }
+                        }
+                    }
+                    if (mindist1 <= srange2) {
+                        BOOLVEC_SET1(p->within_srange, pi);
+                    }
+                }
             }
         }
     }
@@ -369,6 +422,16 @@ void game_update_within_range(struct game_s *g)
     }
 }
 
+void game_update_reloc_dest(struct game_s *g)
+{
+    for (int i = 0; i < g->galaxy_stars; ++i) {
+        struct planet_s *pf = &(g->planet[i]);
+        if (!game_reloc_dest_ok(g, pf->reloc, pf->owner)) {
+            pf->reloc = i;
+        }
+    }
+}
+
 void game_update_empire_contact(struct game_s *g)
 {
     uint8_t tbl_pnum[PLAYER_NUM];
@@ -628,7 +691,10 @@ int game_get_pop_growth_max(const struct game_s *g, const planet_t *p, int max_p
             v *= 2;
         }
     }
-    retval = (v * p->pop + 5) / 100 + p->pop_tenths;
+    retval = (v * p->pop + 5) / 100;
+    if (!game_num_pop_tenths_fix) {
+        retval += p->pop_tenths;
+    }
     if ((v > 0) && (retval < 1)) {
         retval = 1;
     }
@@ -671,14 +737,14 @@ int game_get_tech_prod(int prod, int slider, race_t race, planet_special_t speci
     return MIN(v, 0x7fff);
 }
 
-void game_print_prod_of_total(const struct game_s *g, player_id_t pi, int prod, char *buf)
+void game_print_prod_of_total(const struct game_s *g, player_id_t pi, int prod, char *buf, size_t bufsize)
 {
     int v = g->eto[pi].total_production_bc;
     if (v == 0) {
-        strcpy(buf, "0%%");
+        lib_strcpy(buf, "0%%", bufsize);
     } else {
         v = (prod * 1000) / v;
-        sprintf(buf, "%i.%i%%", v / 10, v % 10);
+        lib_sprintf(buf, bufsize, "%i.%i%%", v / 10, v % 10);
     }
 }
 
@@ -699,7 +765,7 @@ bool game_xy_is_in_nebula(const struct game_s *g, int x, int y)
     return false;
 }
 
-int game_calc_eta(const struct game_s *g, int speed, int x0, int y0, int x1, int y1)
+int game_calc_eta_ship(const struct game_s *g, int speed, int x0, int y0, int x1, int y1)
 {
     int x, y, num = 0;
     x = x0;
@@ -710,13 +776,30 @@ int game_calc_eta(const struct game_s *g, int speed, int x0, int y0, int x1, int
         v = speed;
         first = true;
         do {
+            /* Inside nebulas, ships move on frame 0 and then on every odd frame. */
             if (first || (!game_xy_is_in_nebula(g, x, y))) {
                 util_math_go_line_dist(&x, &y, x1, y1, 5);
             }
-            if ((x != x1) || (y != y1)) {
+            util_math_go_line_dist(&x, &y, x1, y1, 6);
+            first = false;
+            --v;
+        } while (v > 0);
+        ++num;
+    }
+    return num;
+}
+
+int game_calc_eta_trans(const struct game_s *g, int speed, int x0, int y0, int x1, int y1)
+{
+    int x = x0, y = y0, num = 0;
+    while ((x != x1) || (y != y1)) {
+        int v = speed;
+        do {
+            /* Inside nebulas, transports move on every even frame. */
+            util_math_go_line_dist(&x, &y, x1, y1, 5);
+            if (!game_xy_is_in_nebula(g, x, y)) {
                 util_math_go_line_dist(&x, &y, x1, y1, 6);
             }
-            first = false;
             --v;
         } while (v > 0);
         ++num;
@@ -737,6 +820,11 @@ bool game_transport_dest_ok(const struct game_s *g, const planet_t *p, player_id
     }
 }
 
+bool game_reloc_dest_ok(const struct game_s *g, uint8_t planet_i, player_id_t pi)
+{
+    return game_num_extended_reloc_range ? (g->planet[planet_i].within_frange[pi] == 1) : (g->planet[planet_i].owner == pi);
+}
+
 void game_rng_step(struct game_s *g)
 {
     /* TODO disable for multiplayer */
@@ -744,3 +832,108 @@ void game_rng_step(struct game_s *g)
         rnd_0_nm1(2, &g->seed);
     }
 }
+
+/* Max. population values: Each planet has three max_pop variables which are
+ * affected by the ECO projects in different ways.
+ *
+ * The base size, max_pop1, is the size of the planet at the start of the game.
+ * It is used to determine the increase in max. population from (advanced) soil
+ * enrichment; max_pop1 itself is increased only when applying atmospheric
+ * terraforming to some types of hostile planets.
+ *
+ * max_pop2 is max_pop1 modified by (advanced) soil enrichment. Note that for
+ * planets that are Fertile or Gaia at the start of the game,
+ * max_pop1 == max_pop2. They are only different when the planets were made
+ * Fertile or Gaia by a player (human or AI).
+ *
+ * max_pop3 is the actual population maximum. It equals max_pop2 modified by
+ * terraforming and/or by bioweapon damage.
+ */
+void game_turn_atmos_tform(struct planet_s *p) {
+    int max_pop_increase;
+    if (p->type < PLANET_TYPE_DEAD) {
+        max_pop_increase = 20;
+    } else if (p->type < PLANET_TYPE_BARREN) {
+        max_pop_increase = 10;
+    } else {
+        max_pop_increase = 0;
+    }
+    /* WASBUG max_pop += moved from outside if (bc >= cost) */
+    p->type = PLANET_TYPE_MINIMAL;
+    p->max_pop1 += max_pop_increase;
+    p->max_pop2 += max_pop_increase;
+    p->max_pop3 += max_pop_increase;
+    p->growth = PLANET_GROWTH_NORMAL;
+}
+
+void game_turn_soil_enrich(struct planet_s *p, int best_tform, bool advanced) {
+    int max_pop_increase = 0;
+    int16_t old_max_pop2 = 0;
+    p->growth = advanced ? PLANET_GROWTH_GAIA : PLANET_GROWTH_FERTILE;
+    if (advanced) {
+        max_pop_increase = (p->max_pop1 / 10) * 5;
+        /* BUG? If we want to calculate 50% of the base size, rounded up
+        * to the next multiple of 5, we'd check if p->max_pop1 % 10 != 0
+        * below. Instead, we add an additional +5 unless the base size is
+        * in the range 50-59 or 100-109. Penalizing these specific sizes
+        * seems weird and arbitrary. */
+        if ((p->max_pop1 / 10) % 5) {
+            max_pop_increase += 5;
+        }
+        if (game_num_soil_rounding_fix) {
+            max_pop_increase = ((p->max_pop1 + 9) / 10) * 5;
+        }
+    } else {
+        max_pop_increase = (p->max_pop1 / 20) * 5;
+        /* BUG? See above. */
+        if ((p->max_pop1 / 20) % 5) {
+            max_pop_increase += 5;
+        }
+        if (game_num_soil_rounding_fix) {
+            max_pop_increase = ((p->max_pop1 + 19) / 20) * 5;
+        }
+    }
+    SETMAX(max_pop_increase, 5);
+    old_max_pop2 = p->max_pop2;
+    p->max_pop2 = p->max_pop1 + max_pop_increase;
+    p->max_pop3 += p->max_pop2 - old_max_pop2;
+    if (game_num_reset_tform_to_max) {
+        /* BUG? Having p->max_pop3 > (best_tform + p->max_pop2) seems
+         * only possible when conquering a planet from a race with better
+         * terraforming tech. Is it intended that the player loses the
+         * more advanced terraforming here, or is this a sanity check
+         * with unintended consequences? */
+        SETMIN(p->max_pop3, (best_tform + p->max_pop2));
+    }
+    SETMIN(p->max_pop3, game_num_max_pop);
+}
+
+void game_ship_build_everywhere(struct game_s *g, player_id_t owner, uint8_t ship_i) {
+    for (uint8_t i = 0; i < g->galaxy_stars; ++i) {
+        planet_t *p = &(g->planet[i]);
+        if (p->owner == owner) {
+            if (ship_i == BUILDSHIP_STARGATE) {
+                if (!p->have_stargate) {
+                    p->buildship = BUILDSHIP_STARGATE;
+                }
+            } else {
+                p->buildship = ship_i;
+            }
+        }
+    }
+}
+
+void game_ship_replace_everywhere(struct game_s *g, player_id_t owner, uint8_t replace_i, uint8_t ship_i) {
+    for (uint8_t i = 0; i < g->galaxy_stars; ++i) {
+        planet_t *p = &(g->planet[i]);
+        if (p->owner == owner && p->buildship == replace_i) {
+            if (ship_i == BUILDSHIP_STARGATE) {
+                if (!p->have_stargate) {
+                    p->buildship = BUILDSHIP_STARGATE;
+                }
+            } else {
+                p->buildship = ship_i;
+            }
+        }
+    }
+}
