@@ -3,6 +3,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "game_nebula.h"
 #include "game_new.h"
 #include "comp.h"
 #include "game.h"
@@ -27,69 +28,6 @@
 #define EMPEROR_NAMES_PER_RACE  6
 #define EMPEROR_NAME_LBX_LEN    20
 
-static const uint8_t tbl_nebula_data[NEBULA_TYPE_NUM][4][4] = {
-    { /* 0 */
-        { 0x14, 0x07, 0x0f, 0x1f },
-        { 0x05, 0x12, 0x0c, 0x02 },
-        { 0x28, 0x1c, 0x1f, 0x24 },
-        { 0x14, 0x27, 0x1b, 0x04 }
-    },
-    { /* 1 */
-        { 0x0a, 0x14, 0x1b, 0x1e },
-        { 0x11, 0x0d, 0x0d, 0x05 },
-        { 0x1a, 0x26, 0x29, 0x29 },
-        { 0x25, 0x18, 0x10, 0x10 }
-    },
-    { /* 2 */
-        { 0x09, 0x03, 0x17, 0x1a },
-        { 0x02, 0x08, 0x12, 0x0f },
-        { 0x16, 0x19, 0x29, 0x23 },
-        { 0x19, 0x15, 0x20, 0x24 }
-    },
-    { /* 3 */
-        { 0x19, 0x08, 0x0a, 0x10 },
-        { 0x0c, 0x0f, 0x09, 0x07 },
-        { 0x1e, 0x1c, 0x16, 0x17 },
-        { 0x1c, 0x1d, 0x0f, 0x09 }
-    },
-    { /* 4 */
-        { 0x07, 0x0f, 0x15, 0x19 },
-        { 0x02, 0x0f, 0x16, 0x23 },
-        { 0x19, 0x25, 0x29, 0x29 },
-        { 0x13, 0x1c, 0x23, 0x27 }
-    },
-    { /* 5 */
-        { 0x03, 0x08, 0x0e, 0x09 },
-        { 0x0e, 0x15, 0x02, 0x0a },
-        { 0x23, 0x24, 0x1a, 0x22 },
-        { 0x14, 0x23, 0x1f, 0x0f }
-    },
-    { /* 6 */
-        { 0x0d, 0x09, 0x13, 0x05 },
-        { 0x03, 0x0b, 0x21, 0x16 },
-        { 0x1b, 0x22, 0x1b, 0x08 },
-        { 0x21, 0x20, 0x25, 0x1e }
-    },
-    { /* 7 */
-        { 0x08, 0x0f, 0x1c, 0x25 },
-        { 0x08, 0x18, 0x02, 0x0a },
-        { 0x25, 0x22, 0x23, 0x27 },
-        { 0x19, 0x1f, 0x07, 0x12 }
-    },
-    { /* 8 */
-        { 0x08, 0x05, 0x05, 0x1b },
-        { 0x06, 0x01, 0x0c, 0x03 },
-        { 0x23, 0x0c, 0x09, 0x25 },
-        { 0x1f, 0x08, 0x15, 0x0a }
-    },
-    { /* 9 */
-        { 0x04, 0x08, 0x0d, 0x25 },
-        { 0x02, 0x12, 0x0e, 0x11 },
-        { 0x2a, 0x26, 0x26, 0x29 },
-        { 0x12, 0x1e, 0x21, 0x1d }
-    }
-};
-
 /* index to planets.lbx + 1 */
 /* index to starview.lbx - 6 */
 static const uint8_t tbl_planet_type_infogfx[PLANET_TYPE_NUM - 1][6] = {
@@ -199,6 +137,8 @@ static void game_generate_planets(struct game_s *g)
 
         p->look = rnd_0_nm1(2, &g->seed) * 6;
         p->frame = rnd_0_nm1(50, &g->seed);
+        /* Needed for the original sequence of random numbers */
+        /*p->field_16 =*/ rnd_0_nm1(4, &g->seed);
 
         in_nebula = false;
         for (int k = 0; k < g->nebula_num; ++k) {
@@ -248,7 +188,7 @@ static void game_generate_planets(struct game_s *g)
                     if ((r > 0xc) && (r < 0x10)) { t = PLANET_TYPE_ARID; }
                     if ((r > 0xf) && (r < 0x12)) { t = PLANET_TYPE_OCEAN; }
                     if ((r > 0x11) && (r < 0x14)) { t = PLANET_TYPE_JUNGLE; }
-                    if (r > 0x14) { t = PLANET_TYPE_TERRAN; }
+                    if (r >= 0x14) { t = PLANET_TYPE_TERRAN; }
                     break;
                 case STAR_TYPE_GREEN:
                     /*if (r < 2) { t = PLANET_TYPE_NOT_HABITABLE; }*/
@@ -377,9 +317,11 @@ static void game_generate_planets(struct game_s *g)
                 if (star_type == STAR_TYPE_RED) { di -= 4; } else if (star_type == STAR_TYPE_GREEN) { di -= 2; }
                 if (di <= 2) {
                     p->special = PLANET_SPECIAL_POOR;
-                    di = rnd_1_n(0x14, &g->seed);
-                    if (star_type == STAR_TYPE_RED) { di -= 4; } else if (star_type == STAR_TYPE_GREEN) { di -= 2; }
-                    if (di <= 5) {
+                }
+                di = rnd_1_n(0x14, &g->seed);
+                if (star_type == STAR_TYPE_RED) { di -= 4; } else if (star_type == STAR_TYPE_GREEN) { di -= 2; }
+                if (di <= 5) {
+                    if (p->special == PLANET_SPECIAL_POOR) {
                         p->special = PLANET_SPECIAL_ULTRA_POOR;
                     }
                 }
@@ -388,9 +330,11 @@ static void game_generate_planets(struct game_s *g)
             if (star_type == STAR_TYPE_BLUE) { di -= 2; } else if (star_type == STAR_TYPE_NEUTRON) { di -= 5; }
             if ((((int)PLANET_TYPE_STEPPE) - ((int)p->type)) > di) {
                 p->special = PLANET_SPECIAL_RICH;
-                di = rnd_1_n(0x14, &g->seed) - (in_nebula ? 8 : 0);
-                if (star_type == STAR_TYPE_BLUE) { di -= 2; } else if (star_type == STAR_TYPE_NEUTRON) { di -= 5; }
-                if (di < 6) {
+            }
+            di = rnd_1_n(0x14, &g->seed) - (in_nebula ? 8 : 0);
+            if (star_type == STAR_TYPE_BLUE) { di -= 2; } else if (star_type == STAR_TYPE_NEUTRON) { di -= 5; }
+            if (di < 6) {
+                if (p->special == PLANET_SPECIAL_RICH) {
                     p->special = PLANET_SPECIAL_ULTRA_RICH;
                 }
             }
@@ -482,7 +426,7 @@ static void game_generate_planets(struct game_s *g)
     }
 }
 
-static void game_generate_galaxy(struct game_s *g)
+static void game_generate_galaxy(struct game_s *g, struct game_new_options_s *opts)
 {
     {
         uint16_t a = 0, w = 0, h = 0;
@@ -511,7 +455,7 @@ static void game_generate_galaxy(struct game_s *g)
                 break;
         }
 
-        g->nebula_num = a;
+        g->nebula_num = opts->nebulae ? a : 0;
         g->galaxy_w = w;
         g->galaxy_h = h;
         g->galaxy_maxx = (w - 1) * 0x1c + 0x14;
@@ -571,53 +515,46 @@ static void game_generate_galaxy(struct game_s *g)
 
 static void game_generate_planet_names(struct game_s *g)
 {
-    BOOLVEC_DECLARE(in_use, PLANET_NAMES_MAX);
-    BOOLVEC_CLEAR(in_use, PLANET_NAMES_MAX);
+    BOOLVEC_DECLARE(in_use, PLANETS_MAX);
+    BOOLVEC_CLEAR(in_use, PLANETS_MAX);
     for (int i = 0; i < g->galaxy_stars; ++i) {
         uint16_t j;
-        j = rnd_0_nm1(PLANET_NAMES_MAX, &g->seed);
-        while (BOOLVEC_IS1(in_use, j)) {
-            if (++j == PLANET_NAMES_MAX) { j = 0; }
-        }
+        do {
+            j = rnd_0_nm1(PLANETS_MAX, &g->seed);
+        } while (BOOLVEC_IS1(in_use, j));
         BOOLVEC_SET1(in_use, j);
-        strcpy(g->planet[i].name, game_str_tbl_planet_names[j]);
+        lib_strcpy(g->planet[i].name, game_str_tbl_planet_names[j], PLANET_NAME_LEN);
     }
-    strcpy(g->planet[g->evn.planet_orion_i].name, game_str_planet_name_orion);
+    lib_strcpy(g->planet[g->evn.planet_orion_i].name, game_str_planet_name_orion, PLANET_NAME_LEN);
 }
 
 static void game_generate_race_banner(struct game_s *g)
 {
-    BOOLVEC_DECLARE(in_use, MAX((int)RACE_NUM, (int)BANNER_NUM));
+    const uint16_t race_tbl[] = {7, 1, 5, 0, 3, 9, 4, 6, 2, 8};
+    BOOLVEC_DECLARE(in_use, (int)BANNER_NUM);
     uint16_t loops;
     loops = 0;
-    BOOLVEC_CLEAR(in_use, MAX((int)RACE_NUM, (int)BANNER_NUM));
     for (player_id_t i = PLAYER_0; i < g->players; ++i) {
         race_t race;
+        bool in_use = false;
         race = g->eto[i].race;
         if (race != RACE_RANDOM) {
-            BOOLVEC_SET1(in_use, race);
             continue;
         }
-        race = rnd_0_nm1(RACE_NUM, &g->seed);
-        if (BOOLVEC_IS0(in_use, race)) {
-            BOOLVEC_SET1(in_use, race);
-            g->eto[i].race = race;
-        } else {
-            --i; /* try again */
-            if (++loops == 100) {
-                for (race = 0; race < RACE_NUM; ++race) {
-                    if (BOOLVEC_IS0(in_use, race)) {
-                        BOOLVEC_SET1(in_use, race);
-                        g->eto[i].race = race;
-                        break;
-                    }
-                }
-                loops = 0;
+        race = race_tbl[rnd_0_nm1(RACE_NUM, &g->seed)];
+        for (player_id_t j = PLAYER_0; j < i; ++j) {
+            if (g->eto[j].race == race) {
+                in_use = true;
+                --i;
+                break;
             }
         }
+        if (!in_use) {
+            g->eto[i].race = race;
+        }
     }
     loops = 0;
-    BOOLVEC_CLEAR(in_use, MAX((int)RACE_NUM, (int)BANNER_NUM));
+    BOOLVEC_CLEAR(in_use, (int)BANNER_NUM);
     for (player_id_t i = PLAYER_0; i < g->players; ++i) {
         banner_t banner;
         banner = g->eto[i].banner;
@@ -647,20 +584,99 @@ static void game_generate_race_banner(struct game_s *g)
         if (IS_AI(g, i)) {
             race_t r;
             r = g->eto[i].race;
-            g->eto[i].trait1 = game_num_tbl_trait1[r][rnd_0_nm1(TRAIT1_TBL_NUM, &g->seed)];
             g->eto[i].trait2 = game_num_tbl_trait2[r][rnd_0_nm1(TRAIT2_TBL_NUM, &g->seed)];
         }
     }
+    for (player_id_t i = PLAYER_0; i < g->players; ++i) {
+        race_t r;
+        r = g->eto[i].race;
+        g->eto[i].trait1 = game_num_tbl_trait1[r][rnd_0_nm1(TRAIT1_TBL_NUM, &g->seed)];
+    }
 }
 
-static void game_generate_home_etc(struct game_s *g)
+static bool game_generate_home_improved_do_i(struct game_s *g, struct game_new_options_s *opts, uint8_t tblhome[], player_id_t i)
+{   //NOTE: Bottleneck for SMALL galaxy and 6 PLAYERS
+    static const int mindist = 80;
+    uint8_t ok_home_i[PLANETS_MAX];
+    uint16_t ok_home_count = 0;
+    int safe_mindist = (g->galaxy_size > GALAXY_SIZE_SMALL || g->players < 6) ? mindist : mindist - 10;
+    for (uint8_t pi = 0; pi < g->galaxy_stars; ++pi) {
+        int x = g->planet[pi].x;
+        int y = g->planet[pi].y;
+        {
+            bool next = false;
+            int num_checks = opts->homeworlds.num_dist_checks;
+            for (int j = 0; j < i; ++j) {
+                planet_t *p;
+                p = &g->planet[tblhome[j]];
+                if (tblhome[j] == pi) {
+                    next = true;
+                    break;
+                }
+                if ((j < num_checks) && (util_math_dist_fast(x, y, p->x, p->y) < safe_mindist)) {
+                    next = true;
+                    break;
+                }
+            }
+            if (next) continue;
+        }
+        if ((i < opts->homeworlds.num_ok_planet_checks) && (i < 5 || g->galaxy_size > GALAXY_SIZE_SMALL)) {
+            bool next = true;
+            for (int j = 0; j < g->galaxy_stars; ++j) {
+                planet_t *p;
+                p = &g->planet[j];
+                if (1
+                    && (p->type >= PLANET_TYPE_MINIMAL)
+                    && (j != g->evn.planet_orion_i)
+                    && (j != pi)
+                    ) {
+                    if (util_math_dist_fast(x, y, p->x, p->y) <= 29) {
+                        next = false;
+                        break;
+                    }
+                }
+            }
+            if (next) continue;
+        }
+        if (pi == g->evn.planet_orion_i) {
+            continue;
+        }
+        ok_home_i[ok_home_count] = pi;
+        ++ok_home_count;
+    }
+    while (ok_home_count > 0) {
+        int k = rnd_0_nm1(ok_home_count, &g->seed);
+        tblhome[i] = ok_home_i[k];
+        --ok_home_count;
+        ok_home_i[k] = ok_home_i[ok_home_count];
+        if ((i + 1) >= g->players) {
+            return true;
+        }
+        if ((i < (PLAYER_NUM - 1)) && game_generate_home_improved_do_i(g, opts, tblhome, i + 1)) {
+            return true;
+        }
+    }
+    return false;
+}
+
+static bool game_generate_home_improved_do(struct game_s *g, struct game_new_options_s *opts, uint16_t tblhome[])
 {
-    uint16_t tblhome[PLAYER_NUM];
-    uint16_t homei, loops;
+    uint8_t homes[PLAYER_NUM] = { PLANET_NONE , PLANET_NONE , PLANET_NONE , PLANET_NONE , PLANET_NONE , PLANET_NONE };
+    game_generate_race_banner(g);
+    bool result = game_generate_home_improved_do_i(g, opts, homes, PLAYER_0);
+    for (int i = 0; i < PLAYER_NUM; ++i) {
+        tblhome[i] = homes[i];
+    }
+    return result;
+}
+
+static bool game_generate_home_do(struct game_s *g, uint16_t tblhome[])
+{
+    uint16_t loops;
     bool flag_all_ok;
-start_of_func:
     flag_all_ok = false;
     loops = 0;
+    game_generate_race_banner(g);
     while ((!flag_all_ok) && (loops < 200)) {
         flag_all_ok = true;
         for (player_id_t i = PLAYER_0; i < g->players; ++i) {
@@ -764,12 +780,23 @@ start_of_func:
         }
     }
 #endif
-    if (!flag_all_ok) {
-        game_generate_galaxy(g);
+    return flag_all_ok;
+}
+
+static void game_generate_home_etc(struct game_s *g, struct game_new_options_s *opts)
+{
+    uint16_t tblhome[PLAYER_NUM];
+    uint16_t homei;
+    bool flag_all_ok;
+    do {
+        game_generate_galaxy(g, opts);
         game_generate_planet_names(g);
-        goto start_of_func;
-    }
-    game_generate_race_banner(g);   /* must be run once and before the home planet name copy below */
+        if (opts->improved_galaxy_generator) {
+            flag_all_ok = game_generate_home_improved_do(g, opts, tblhome);
+        } else {
+            flag_all_ok = game_generate_home_do(g, tblhome);
+        }
+    } while (!flag_all_ok);
     for (player_id_t i = PLAYER_0; i < g->players; ++i) {
         planet_t *p;
         homei = tblhome[i];
@@ -780,12 +807,18 @@ start_of_func:
         p->claim = i;
         p->type = PLANET_TYPE_TERRAN;
         p->star_type = STAR_TYPE_YELLOW;
-        p->max_pop2 = 100;
-        p->max_pop3 = 100;
-        p->max_pop1 = 100;
+        if ((opts->homeworlds.max_pop < 50) || (opts->homeworlds.max_pop > 120)) {
+            opts->homeworlds.max_pop = 100;
+        }
+        p->max_pop2 = opts->homeworlds.max_pop;
+        p->max_pop3 = opts->homeworlds.max_pop;
+        p->max_pop1 = opts->homeworlds.max_pop;
         p->infogfx = tbl_planet_type_infogfx[p->type][rnd_0_nm1(6, &g->seed)] - 1;
         p->growth = PLANET_GROWTH_NORMAL;
-        p->special = PLANET_SPECIAL_NORMAL;
+        if ((opts->homeworlds.special < PLANET_SPECIAL_ULTRA_POOR) || (opts->homeworlds.special >= PLANET_SPECIAL_NUM)) {
+            opts->homeworlds.special = PLANET_SPECIAL_NORMAL;
+        }
+        p->special = opts->homeworlds.special;
         BOOLVEC_SET1(p->explored, i);
         BOOLVEC_SET1(p->within_srange, i);
         p->within_frange[i] = 1;
@@ -808,7 +841,7 @@ start_of_func:
                 break;
         }
         p->pop_prev = p->pop;
-        game_new_generate_home_name(g->eto[i].race, p->name);
+        game_new_generate_home_name(g->eto[i].race, p->name, PLANET_NAME_LEN);
     }
     for (int j = 0; j < g->galaxy_stars; ++j) {
         player_id_t owner;
@@ -818,6 +851,18 @@ start_of_func:
         }
         /* seen_pop.._factories cleared here */
     }
+    if ((opts->homeworlds.num_scouts < 0) || (opts->homeworlds.num_scouts > 5)) {
+        opts->homeworlds.num_scouts = 2;
+    }
+    startfleet_ships[0] = opts->homeworlds.num_scouts;
+    if ((opts->homeworlds.num_fighters < 0) || (opts->homeworlds.num_fighters > 10)) {
+        opts->homeworlds.num_fighters = 0;
+    }
+    startfleet_ships[1] = opts->homeworlds.num_fighters;
+    if ((opts->homeworlds.num_colony_ships < 0) || (opts->homeworlds.num_colony_ships > 2)) {
+        opts->homeworlds.num_colony_ships = 1;
+    }
+    startfleet_ships[4] = opts->homeworlds.num_colony_ships;
     for (player_id_t i = PLAYER_0; i < g->players; ++i) {
         empiretechorbit_t *e;
         e = &g->eto[i];
@@ -837,7 +882,7 @@ start_of_func:
             e->shipdesigns_num = startship_num;
             for (int j = 0; j < startship_num; ++j, ++sd) {
                 *sd = tbl_startship[j];
-                strcpy(sd->name, game_str_tbl_stship_names[j]);
+                lib_strcpy(sd->name, game_str_tbl_stship_names[j], SHIP_NAME_LEN);
                 sd->look += e->banner * SHIP_LOOK_PER_BANNER;
             }
             memcpy(&g->current_design[i], &srd->design[0], sizeof(shipdesign_t));
@@ -846,6 +891,10 @@ start_of_func:
                 n = startfleet_ships[j];
                 e->orbit[tblhome[i]].ships[j] = n;
                 srd->shipcount[j] = n;
+                if ((j == 4) && opts->homeworlds.armed_colony_ships && !srd->design[4].wpnt[0]) {
+                    srd->design[4].wpnn[0] = 1;
+                    srd->design[4].wpnt[0] = 2;
+                }
             }
         }
         e->fuel_range = 3;
@@ -989,7 +1038,6 @@ static void game_generate_misc(struct game_s *g)
             e->attack_bounty[pli2] = PLAYER_NONE;
             e->bounty_collect[pli2] = PLAYER_NONE;
         }
-        g->evn.msg_filter[pli][0] = IS_HUMAN(g, pli) ? FINISHED_DEFAULT_FILTER : 0;
     }
 }
 
@@ -1000,6 +1048,7 @@ static void game_generate_load(struct game_s *g)
     game_update_tech_util(g);
     for (player_id_t pli = PLAYER_0; pli < g->players; ++pli) {
         game_update_eco_on_waste(g, pli, false);
+        game_update_seen_by_orbit(g, pli);
     }
     game_update_within_range(g);
     game_update_visibility(g);
@@ -1019,8 +1068,8 @@ static void game_generate_emperor_names(struct game_s *g, const uint8_t *namedat
             const char *str;
             bool flag_name_used;
             str = (const char *)&namedata[4 + (base + rnd_0_nm1(EMPEROR_NAMES_PER_RACE, &g->seed)) * EMPEROR_NAME_LBX_LEN];
-            strcpy(buf, str);
-            util_trim_whitespace(buf); /* fix "Zygot  " */
+            lib_strcpy(buf, str, sizeof(buf));
+            util_trim_whitespace(buf, sizeof(buf)); /* fix "Zygot  " */
             flag_name_used = false;
             for (player_id_t i = PLAYER_0; i < g->players; ++i) {
                 if (strcasecmp(buf, g->emperor_names[i]) == 0) {
@@ -1029,7 +1078,7 @@ static void game_generate_emperor_names(struct game_s *g, const uint8_t *namedat
                 }
             }
             if (!flag_name_used) {
-                strcpy(g->emperor_names[pli], buf);
+                lib_strcpy(g->emperor_names[pli], buf, EMPEROR_NAME_LEN);
                 break;
             }
         }
@@ -1087,27 +1136,25 @@ int game_new(struct game_s *g, struct game_aux_s *gaux, struct game_new_options_
         }
         log_message("Game: new game -new %u:0x%x:%u:0x%x:%u -nga %u\n", vo, vr, vb, g->galaxy_seed, va, g->ai_id);
     }
-    game_generate_galaxy(g);
-    game_generate_planet_names(g);
-    game_generate_home_etc(g);
+    game_generate_home_etc(g, opt);
     game_generate_relation_etc(g);
     game_generate_research(g, researchflag);
     game_generate_misc(g);
-    game_generate_load(g);
+    if (!game_num_update_on_load) {
+        game_generate_load(g);
+    }
     for (player_id_t i = PLAYER_0; i < g->players; ++i) {
         char *b;
         const char *str;
         str = opt->pdata[i].playername;
         b = g->emperor_names[i];
-        strncpy(b, str, EMPEROR_NAME_LEN);
-        b[EMPEROR_NAME_LEN - 1] = '\0';
-        util_trim_whitespace(b);
+        lib_strcpy(b, str, EMPEROR_NAME_LEN);
+        util_trim_whitespace(b, EMPEROR_NAME_LEN);
         str = opt->pdata[i].homename;
         if (*str != '\0') {
             b = g->planet[g->planet_focus_i[i]].name;
-            strncpy(b, str, PLANET_NAME_LEN);
-            b[PLANET_NAME_LEN - 1] = '\0';
-            util_trim_whitespace(b);
+            lib_strcpy(b, str, PLANET_NAME_LEN);
+            util_trim_whitespace(b, PLANET_NAME_LEN);
         }
     }
     {
@@ -1127,38 +1174,32 @@ int game_new_tutor(struct game_s *g, struct game_aux_s *gaux)
     opt.difficulty = DIFFICULTY_SIMPLE;
     opt.pdata[PLAYER_0].race = RACE_KLACKON;
     opt.pdata[PLAYER_0].banner = BANNER_WHITE;
-    opt.pdata[PLAYER_1].race = RACE_SAKKRA;
-    opt.pdata[PLAYER_1].banner = BANNER_YELLOW;
-    opt.pdata[PLAYER_2].race = RACE_PSILON;
-    opt.pdata[PLAYER_2].banner = BANNER_BLUE;
-    opt.pdata[PLAYER_3].race = RACE_SILICOID;
-    opt.pdata[PLAYER_3].banner = BANNER_GREEN;
-    strcpy(opt.pdata[PLAYER_0].playername, "Mr Tutor");
-    strcpy(opt.pdata[PLAYER_0].homename, "SOL");
-    opt.galaxy_seed = 0xdeadbeef; /* FIXME find value that gives an easy game */
+    lib_strcpy(opt.pdata[PLAYER_0].playername, "Mr Tutor", EMPEROR_NAME_LEN);
+    lib_strcpy(opt.pdata[PLAYER_0].homename, "SOL", PLANET_NAME_LEN);
+    opt.galaxy_seed = 0xfda3f;
     return game_new(g, gaux, &opt);
 }
 
-void game_new_generate_emperor_name(race_t race, char *buf)
+void game_new_generate_emperor_name(race_t race, char *buf, size_t bufsize)
 {
     if (race == RACE_RANDOM) {
-        strcpy(buf, game_str_rndempname);
+        lib_strcpy(buf, game_str_rndempname, bufsize);
     } else {
         uint32_t seed = rnd_get_new_seed();
         uint8_t *namedata = lbxfile_item_get(LBXFILE_NAMES, 0);
         int base = race * EMPEROR_NAMES_PER_RACE;
         const char *str = (const char *)&namedata[4 + (base + rnd_0_nm1(EMPEROR_NAMES_PER_RACE, &seed)) * EMPEROR_NAME_LBX_LEN];
         /* TODO check if in use for the case of forced same races */
-        strcpy(buf, str);
-        util_trim_whitespace(buf); /* fix "Zygot  " */
+        lib_strcpy(buf, str, bufsize);
+        util_trim_whitespace(buf, bufsize); /* fix "Zygot  " */
         lbxfile_item_release(LBXFILE_NAMES, namedata);
     }
 }
 
-void game_new_generate_home_name(race_t race, char *buf)
+void game_new_generate_home_name(race_t race, char *buf, size_t bufsize)
 {
     /* TODO check if in use for the case of forced same races */
-    strcpy(buf, game_str_tbl_home_names[race]);
+    lib_strcpy(buf, game_str_tbl_home_names[race], bufsize);
 }
 
 void game_new_generate_other_emperor_name(struct game_s *g, player_id_t player)
@@ -1170,8 +1211,8 @@ void game_new_generate_other_emperor_name(struct game_s *g, player_id_t player)
         char buf[EMPEROR_NAME_LBX_LEN + 1];
         bool flag_in_use;
         str = (const char *)&namedata[4 + (base + rnd_0_nm1(EMPEROR_NAMES_PER_RACE, &g->seed)) * EMPEROR_NAME_LBX_LEN];
-        strcpy(buf, str);
-        util_trim_whitespace(buf); /* fix "Zygot  " */
+        lib_strcpy(buf, str, sizeof(buf));
+        util_trim_whitespace(buf, sizeof(buf)); /* fix "Zygot  " */
         flag_in_use = true;
         for (int i = 0; i < g->players; ++i) {
             if (strcasecmp(g->emperor_names[i], buf) == 0) {
@@ -1180,7 +1221,7 @@ void game_new_generate_other_emperor_name(struct game_s *g, player_id_t player)
             }
         }
         if (!flag_in_use) {
-            strcpy(g->emperor_names[player], buf);
+            lib_strcpy(g->emperor_names[player], buf, EMPEROR_NAME_LEN);
             break;
         }
     }
