@@ -8,6 +8,7 @@
 
 bool game_num_deterministic = false;
 bool game_num_ai_trans_range_fix = false;
+bool game_num_ai_4_colony_curse_fix = false;
 int game_num_bt_turn_max = 50;
 bool game_num_bt_wait_no_reload = false;
 bool game_num_bt_precap_tohit = false;
@@ -57,7 +58,7 @@ bool game_num_waste_adjust_fix = false;
 bool game_num_slider_respects_locks = false;
 bool game_num_pop_tenths_fix = false;
 bool game_num_factory_cost_fix = false;
-bool game_num_doom_stack_fix = true;
+bool game_num_doom_stack_fix = false;
 uint8_t game_num_eco_slider_slack = 7;
 bool game_num_reset_tform_to_max = true;
 bool game_num_soil_rounding_fix = false;
