@@ -13,7 +13,6 @@
 #include "game_aux.h"
 #include "game_battle.h"
 #include "game_bomb.h"
-#include "game_debug.h"
 #include "game_design.h"
 #include "game_diplo.h"
 #include "game_election.h"
@@ -29,6 +28,7 @@
 #include "game_stat.h"
 #include "game_str.h"
 #include "game_tech.h"
+#include "lib.h"
 #include "log.h"
 #include "rnd.h"
 #include "types.h"
@@ -38,6 +38,8 @@
 
 /* -------------------------------------------------------------------------- */
 
+int copyprot_status = 0;
+
 static void game_turn_limit_ships(struct game_s *g)
 {
     for (int ei = 0; ei < g->enroute_num; ++ei) {
@@ -172,11 +174,6 @@ static inline void game_add_planet_to_eco_finished(struct game_s *g, uint8_t pli
     BOOLVEC_SET1(g->planet[pli].finished, FINISHED_SOILATMOS);
 }
 
-static inline void game_add_planet_to_terraf_finished(struct game_s *g, uint8_t pli, player_id_t owner)
-{
-    BOOLVEC_SET1(g->planet[pli].finished, FINISHED_TERRAF);
-}
-
 static void game_turn_build_eco(struct game_s *g)
 {
     for (int i = 0; i < g->galaxy_stars; ++i) {
@@ -188,7 +185,7 @@ static void game_turn_build_eco(struct game_s *g)
             const empiretechorbit_t *e;
             int ecorestore, ecoprod, operating_factories;
             e = &(g->eto[owner]);
-            operating_factories = p->pop * e->colonist_oper_factories;
+            operating_factories = p->pop * game_planet_get_pop_oper_fact(g, p);
             SETMIN(operating_factories, p->factories);
             {
                 int v;
@@ -233,22 +230,12 @@ static void game_turn_build_eco(struct game_s *g)
             if ((ecoprod > 0) && e->have_atmos_terra && (p->growth == PLANET_GROWTH_HOSTILE)) {
                 p->bc_to_ecoproj += ecoprod;
                 if (p->bc_to_ecoproj >= game_num_atmos_cost) {
-                    int v;
-                    if (p->type < PLANET_TYPE_DEAD) {
-                        v = 20;
-                    } else if (p->type < PLANET_TYPE_BARREN) {
-                        v = 10;
-                    } else {
-                        v = 0;
-                    }
-                    /* WASBUG max_pop += moved from outside if (bc >= cost) */
-                    p->type = PLANET_TYPE_MINIMAL;
-                    p->max_pop2 += v;
-                    p->max_pop1 += v;
-                    p->max_pop3 += v;
-                    p->growth = PLANET_GROWTH_NORMAL;
+                    game_turn_atmos_tform(p);
                     ecoprod = p->bc_to_ecoproj - game_num_atmos_cost;
-                    p->bc_to_ecoproj -= game_num_atmos_cost;
+                    p->bc_to_ecoproj = 0;
+                    /* WASBUG MOO1 did p->bc_to_ecoproj -= game_num_atmos_cost
+                     * instead which allowed double-spending of credits if
+                     * p->bc_to_ecoproj > game_num_atmos_cost. */
                     game_add_planet_to_eco_finished(g, i, owner);
                 } else {
                     ecoprod = 0;
@@ -257,20 +244,13 @@ static void game_turn_build_eco(struct game_s *g)
             if ((ecoprod > 0) && e->have_soil_enrich && (p->growth == PLANET_GROWTH_NORMAL)) {
                 p->bc_to_ecoproj += ecoprod;
                 if (p->bc_to_ecoproj > game_num_soil_cost) {
-                    int v;
-                    p->growth = PLANET_GROWTH_FERTILE;
-                    v = (p->max_pop1 / 20) * 5;
-                    if ((p->max_pop1 / 20) % 5) {
-                        v += 5;
-                    }
-                    SETMIN(v, 5);
-                    p->max_pop1 += v;
-                    p->max_pop2 += v;
-                    p->max_pop3 += v;
-                    SETMIN(p->max_pop3, (e->have_terraform_n + p->max_pop2));
-                    SETMIN(p->max_pop3, game_num_max_pop);
+                    game_turn_soil_enrich(p, (int)e->have_terraform_n, false);
                     ecoprod = p->bc_to_ecoproj - game_num_soil_cost;
-                    p->bc_to_ecoproj -= game_num_soil_cost; /* BUG cost was not removed */
+                    /* p->bc_to_ecoproj carries over as partial progress
+                     * towards advanced soil enrichment. */
+                    SETMIN(p->bc_to_ecoproj, game_num_soil_cost);
+                    /* WASBUG MOO1 did not do the SETMIN, allowing
+                     * double-spending again. */
                     game_add_planet_to_eco_finished(g, i, owner);
                 } else {
                     ecoprod = 0;
@@ -279,20 +259,12 @@ static void game_turn_build_eco(struct game_s *g)
             if ((ecoprod > 0) && e->have_adv_soil_enrich && (p->growth < PLANET_GROWTH_GAIA) && (p->growth > PLANET_GROWTH_HOSTILE)) {
                 p->bc_to_ecoproj += ecoprod;
                 if (p->bc_to_ecoproj > game_num_adv_soil_cost) {
-                    int v;
-                    p->growth = PLANET_GROWTH_GAIA;
-                    v = (p->max_pop1 / 10) * 5;
-                    if ((p->max_pop1 / 10) % 5) {
-                        v += 5;
-                    }
-                    SETMIN(v, 5);
-                    p->max_pop1 += v;
-                    p->max_pop2 += v;
-                    p->max_pop3 += v;
-                    SETMIN(p->max_pop3, (e->have_terraform_n + p->max_pop2));
-                    SETMIN(p->max_pop3, game_num_max_pop);
+                    game_turn_soil_enrich(p, (int)e->have_terraform_n, true);
                     ecoprod = p->bc_to_ecoproj - game_num_adv_soil_cost;
-                    p->bc_to_ecoproj -= game_num_adv_soil_cost; /* BUG cost was not removed */
+                    p->bc_to_ecoproj = 0;
+                    /* WASBUG MOO1 did not reset p->bc_to_ecoproj to 0, but
+                     * the planet may become hostile due to a random event and
+                     * then this variable becomes relevant again. */
                     game_add_planet_to_eco_finished(g, i, owner);
                 } else {
                     ecoprod = 0;
@@ -305,7 +277,6 @@ static void game_turn_build_eco(struct game_s *g)
                 if (tmax < (p->max_pop3 + v)) {
                     ecoprod -= (tmax - p->max_pop3) * e->terraform_cost_per_inc;
                     p->max_pop3 = tmax;
-                    game_add_planet_to_terraf_finished(g, i, owner);
                 } else {
                     p->max_pop3 += v;
                     ecoprod = 0;
@@ -576,6 +547,8 @@ static void game_turn_build_ship(struct game_s *g)
             uint8_t si;
             prod = game_adjust_prod_by_special((p->slider[PLANET_SLIDER_SHIP] * p->prod_after_maint) / 100, p->special);
             prod += p->bc_to_ship;
+            /* BUG: A 1 BC bonus for having the slider > 0.
+               This bonus is not taken into account by UI */
             if (p->slider[PLANET_SLIDER_SHIP] > 0) {
                 ++prod;
             }
@@ -647,13 +620,12 @@ static void game_turn_reserve(struct game_s *g)
 
 static inline void game_add_planet_to_build_finished(struct game_s *g, uint8_t pli, player_id_t owner, uint8_t type)
 {
-    planet_t *p = &(g->planet[pli]);
-    if (BOOLVEC_TBL_IS1(g->evn.msg_filter, owner, type)) {
-        BOOLVEC_SET1(p->finished, type);
-        ++g->evn.build_finished_num[owner];
-    } else {
-        BOOLVEC_SET0(p->finished, type);
+    if (game_opt_message_filter[type]) {
+        return;
     }
+    planet_t *p = &(g->planet[pli]);
+    BOOLVEC_SET1(p->finished, type);
+    ++g->evn.build_finished_num[owner];
 }
 
 static void game_turn_build_ind(struct game_s *g)
@@ -669,20 +641,37 @@ static void game_turn_build_ind(struct game_s *g)
             int prod, v;
             e = &(g->eto[owner]);
             factold = fact = p->factories;
-            cost = e->factory_adj_cost;
+            cost = game_planet_get_fact_adj_cost(g, p);
             prod = game_adjust_prod_by_special((p->slider[PLANET_SLIDER_IND] * p->prod_after_maint) / 100, p->special);
             prod += p->bc_to_factory;
             bonus = (e->race == RACE_MEKLAR) ? 2 : 0;
             v = e->colonist_oper_factories - p->pop_oper_fact - bonus;
             if (v > 0) {
                 v = v * (e->factory_cost / 2);
+                if (game_num_factory_cost_fix) {
+                    if (e->race != RACE_MEKLAR) {
+                        v = (e->factory_cost * p->factories) / 2;
+                    } else {
+                        v = 0;
+                    }
+                }
             } else {
                 v = 0;
             }
             if ((prod / cost + fact) > (p->pop * p->pop_oper_fact)) {
+                if (game_num_factory_cost_fix && (fact < (p->pop * p->pop_oper_fact))) {
+                    num = p->pop * p->pop_oper_fact - fact;
+                    fact = p->pop * p->pop_oper_fact;
+                    prod -= num * cost;
+                }
                 p->bc_to_refit += prod;
                 if (p->bc_to_refit >= v) {
-                    p->pop_oper_fact = e->colonist_oper_factories;
+                    if (game_num_factory_cost_fix && (e->race != RACE_MEKLAR)) {
+                        ++p->pop_oper_fact;
+                        SETMIN(p->pop_oper_fact, e->colonist_oper_factories);
+                    } else {
+                        p->pop_oper_fact = e->colonist_oper_factories;
+                    }
                     prod = p->bc_to_refit - v;
                     p->bc_to_refit = 0;
                 } else {
@@ -691,7 +680,7 @@ static void game_turn_build_ind(struct game_s *g)
             }
             num = prod / cost;
             p->bc_to_factory = prod % cost;
-            v = p->max_pop3 * e->colonist_oper_factories;
+            v = p->max_pop3 * game_planet_get_pop_oper_fact(g, p);
             if (v < (fact + num)) {
                 v = (fact + num) - v;
                 SETMAX(v, 0);
@@ -700,7 +689,11 @@ static void game_turn_build_ind(struct game_s *g)
             } else {
                 v = 0;
             }
-            e->reserve_bc += (v * cost) / 2;
+            if (game_num_factory_cost_fix && (p->pop_oper_fact < e->colonist_oper_factories)) {
+                p->bc_to_refit += (v * cost) / 2;
+            } else {
+                e->reserve_bc += (v * cost) / 2;
+            }
             fact += num;
             SETMIN(fact, game_num_max_factories);
             p->factories = fact;
@@ -939,13 +932,21 @@ static void game_turn_explore(struct game_s *g)
                     if (flag_do_colonize) {
                         p->owner = i;
                         p->pop = 2;
+                        if (1
+                          && game_num_colonized_factories_fix
+                          && (p->prev_owner != PLAYER_NONE)
+                          && (p->prev_owner != i)) {
+                            if (p->factories != 0) {
+                                p->pop_oper_fact = 1;
+                            } else {
+                                p->pop_oper_fact = 2;
+                            }
+                            p->bc_to_refit = 0;
+                        }
                         --e->orbit[pli].ships[best_colonyship];
                         if ((pli == g->evn.planet_orion_i) && game_num_news_orion) {
                             g->evn.have_orion_conquer = i + 1;
                         }
-                        BOOLVEC_SET0(p->extras, PLANET_EXTRAS_GOVERNOR);
-                        BOOLVEC_SET0(p->extras, PLANET_EXTRAS_GOV_SPEND_REST_SHIP);
-                        BOOLVEC_SET0(p->extras, PLANET_EXTRAS_GOV_SPEND_REST_IND);
                     }
                 }
             }
@@ -953,13 +954,28 @@ static void game_turn_explore(struct game_s *g)
     }
 }
 
-static int game_turn_transport_shoot(struct game_s *g, uint8_t planet_i, player_id_t rowner, uint8_t speed, player_id_t attacker, int bases, weapon_t basewpnt)
+static int game_turn_transport_shoot(struct game_s *g, uint8_t planet_i, player_id_t rowner, uint16_t num_transports, uint8_t speed, player_id_t attacker, int bases, weapon_t basewpnt)
 {
     const planet_t *p = &(g->planet[planet_i]);
     const empiretechorbit_t *ea = &(g->eto[attacker]);
     const empiretechorbit_t *ed = &(g->eto[rowner]);
-    int totaldmg = 0, complevel, killed;
+    int totaldmg = 0, transport_hp, total_hp, complevel, killed;
     uint8_t bestcomp = 0, bestarmor = 0;
+    const uint8_t *rc = &(g->srd[rowner].researchcompleted[TECH_FIELD_CONSTRUCTION][0]);
+
+    /* Transports use the best armor that the owner has researched. */
+    for (int i = 0; i < ed->tech.completed[TECH_FIELD_CONSTRUCTION]; ++i) {
+        uint8_t tier;
+        tech_group_t group;
+        group = game_tech_get_group(g->gaux, TECH_FIELD_CONSTRUCTION, rc[i]);
+        tier = game_tech_get_tier(g->gaux, TECH_FIELD_CONSTRUCTION, rc[i]);
+        if (group == TECH_GROUP_ARMOR) {
+            bestarmor = tier;
+        }
+    }
+    transport_hp = ((bestarmor + 1) * 15);
+    total_hp = transport_hp * num_transports;
+
     uint32_t tbl[WEAPON_NUM];
     memset(tbl, 0, sizeof(tbl));
     tbl[basewpnt] = bases * 24;
@@ -1029,20 +1045,12 @@ static int game_turn_transport_shoot(struct game_s *g, uint8_t planet_i, player_
                     }
                 }
             }
-        }
-    }
-    /*7c521*/
-    {
-        const uint8_t *rc = &(g->srd[rowner].researchcompleted[TECH_FIELD_CONSTRUCTION][0]);
-        for (int i = 0; i < ed->tech.completed[TECH_FIELD_CONSTRUCTION]; ++i) {
-            const uint8_t *r;
-            r = RESEARCH_D0_PTR(g->gaux, TECH_FIELD_CONSTRUCTION, rc[i]);
-            if (r[0] == 7) {
-                bestarmor = r[1];
+            if (totaldmg >= total_hp) {
+                break;
             }
         }
     }
-    killed = totaldmg / ((bestarmor + 1) * 15);
+    killed = totaldmg / transport_hp;
     SETMIN(killed, 1000);
     for (monster_id_t i = MONSTER_CRYSTAL; i <= MONSTER_AMOEBA; ++i) {
         monster_t *m;
@@ -1084,7 +1092,7 @@ static void game_turn_transport(struct game_s *g)
                     empiretechorbit_t *e;
                     e = &(g->eto[j]);
                     if (j == p->owner) {
-                        pop3 -= game_turn_transport_shoot(g, dest, owner, r->speed, j, p->missile_bases, e->base_weapon);
+                        pop3 -= game_turn_transport_shoot(g, dest, owner, pop3, r->speed, j, p->missile_bases, e->base_weapon);
                     } else {
                         /*e102*/
                         bool any_ships;
@@ -1096,7 +1104,7 @@ static void game_turn_transport(struct game_s *g)
                             }
                         }
                         if (any_ships) {
-                            pop3 -= game_turn_transport_shoot(g, dest, owner, r->speed, j, 0, WEAPON_NONE);
+                            pop3 -= game_turn_transport_shoot(g, dest, owner, pop3, r->speed, j, 0, WEAPON_NONE);
                         }
                     }
                 }
@@ -1135,7 +1143,7 @@ static void game_turn_transport(struct game_s *g)
             }
             /*e2a4*/
             if (pop3 <= 0) {
-                if (IS_HUMAN(g, owner) || ((p->owner != PLAYER_NONE) && IS_HUMAN(g, p->owner))) {
+                if (IS_HUMAN(g, owner) || IS_HUMAN(g, p->owner)) {
                     char *buf = ui_get_strbuf();
                     const char *s;
                     if ((g->gaux->local_players == 1) && IS_HUMAN(g, owner)) {
@@ -1143,14 +1151,14 @@ static void game_turn_transport(struct game_s *g)
                     } else {
                         s = game_str_tbl_race[g->eto[owner].race];
                     }
-                    sprintf(buf, "%s %s %s %s", s, game_str_sm_traad1, g->planet[dest].name, game_str_sm_traad2);
+                    lib_sprintf(buf, UI_STRBUF_SIZE, "%s %s %s %s", s, game_str_sm_traad1, g->planet[dest].name, game_str_sm_traad2);
                     ui_turn_msg(g, IS_HUMAN(g, owner) ? owner : p->owner, buf);
                 }
             } else if (p->owner == PLAYER_NONE) {
                 /*e36d*/
                 if (IS_HUMAN(g, owner)) {
                     char *buf = ui_get_strbuf();
-                    sprintf(buf, "%s %s %s", game_str_sm_trbdb1, g->planet[dest].name, game_str_sm_trbdb2);
+                    lib_sprintf(buf, UI_STRBUF_SIZE, "%s %s %s", game_str_sm_trbdb1, g->planet[dest].name, game_str_sm_trbdb2);
                     ui_turn_msg(g, owner, buf);
                 }
             } else {
@@ -1200,7 +1208,7 @@ static void game_turn_coup(struct game_s *g)
         }
     }
     for (player_id_t i = PLAYER_0; i < g->players; ++i) {
-        if ((tbl_rebelplanets[i] > ((tbl_planets[i] + 1) / 2)) && IS_ALIVE(g, i)) {
+        if ((tbl_rebelplanets[i] > tbl_planets[i] / 2) && IS_ALIVE(g, i)) {
             empiretechorbit_t *e = &(g->eto[i]);
             e->trait2 = rnd_0_nm1(TRAIT2_NUM, &g->seed);
             e->trait1 = rnd_0_nm1(TRAIT1_NUM, &g->seed);
@@ -1281,7 +1289,7 @@ static bool game_turn_check_end(struct game_s *g, struct game_end_s *ge)
                 } else if (pi2 == PLAYER_NONE) {
                     pi2 = i;
                 }
-                if (IS_HUMAN(g, i) && (good_planets[i] > 0)) {
+                if (IS_HUMAN(g, i) && (good_planets[i] > 0) && IS_ALIVE(g, i)) {
                     human_alive = true;
                 }
             }
@@ -1408,16 +1416,25 @@ static void game_turn_finished_slider(struct game_s *g)
         planet_t *p = &(g->planet[pli]);
         player_id_t pi;
         empiretechorbit_t *e;
+        bool cond_build_ind;
         pi = p->owner;
         if ((pi == PLAYER_NONE) || IS_AI(g, pi)) {
             BOOLVEC_CLEAR(p->finished, FINISHED_NUM);
             continue;
         }
         e = &(g->eto[pi]);
-        if (BOOLVEC_IS1(p->finished, FINISHED_SOILATMOS)) {
+        cond_build_ind = (p->factories < (p->pop * e->colonist_oper_factories));
+        if (game_num_cond_switch_to_ind_fix) {
+            cond_build_ind = (p->factories < (p->max_pop3 * e->colonist_oper_factories));
+        }
+        if (game_num_factory_cost_fix) {
+            cond_build_ind |= (p->pop_oper_fact < e->colonist_oper_factories);
+        }
+        if (BOOLVEC_IS1(p->finished, FINISHED_SOILATMOS)
+          && !(game_num_slider_eco_done_fix && game_planet_can_terraform(g, p, p->owner, true))) {
             int v;
-            v = game_planet_get_w1(g, pli);
-            if (p->factories >= (p->pop * e->colonist_oper_factories)) {
+            v = game_planet_get_waste_percent(NULL, g, p, false);
+            if (!cond_build_ind) {
                 p->slider[PLANET_SLIDER_TECH] += p->slider[PLANET_SLIDER_ECO] - v;
             } else {
                 p->slider[PLANET_SLIDER_IND] += p->slider[PLANET_SLIDER_ECO] - v;
@@ -1444,23 +1461,15 @@ static void game_turn_finished_slider(struct game_s *g)
                 }
             }
             if (!flag_pending_ecoproj) {
-                int w, fact, waste, prod;
-                fact = p->factories;
-                SETMIN(fact, p->pop * e->colonist_oper_factories);
-                waste = (e->race == RACE_SILICOID) ? 0 : (((fact * e->ind_waste_scale) / 10 + p->waste) / e->have_eco_restoration_n);
-                prod = p->prod_after_maint;
-                if (prod == 0) {
-                    prod = 1000;
-                }
-                w = ((waste * 100) + prod - 1) / prod;
-                SETRANGE(w, 0, 100);
+                int w, waste = 0;
+                w = game_planet_get_waste_percent(&waste, g, p, false);
                 SUBSAT0(v, waste);
                 if (0
                   || ((game_get_pop_growth_for_eco(g, p, v) / 10) > 0)
                   || ((p->slider[PLANET_SLIDER_ECO] != 0) && (e->race == RACE_SILICOID))
                 ) {
                     if ((p->pop > p->pop_prev) || (p->slider[PLANET_SLIDER_ECO] > (w + game_num_eco_slider_slack))) {
-                        if (p->factories >= (p->pop * e->colonist_oper_factories)) {
+                        if (!cond_build_ind) {
                             p->slider[PLANET_SLIDER_TECH] += p->slider[PLANET_SLIDER_ECO] - w;
                         } else {
                             p->slider[PLANET_SLIDER_IND] += p->slider[PLANET_SLIDER_ECO] - w;
@@ -1475,7 +1484,7 @@ static void game_turn_finished_slider(struct game_s *g)
                   && (p->slider[PLANET_SLIDER_ECO] > 0)
                   && ((e->race == RACE_SILICOID) || ((e->ind_waste_scale == 0) && (p->waste == 0)))
                 ) {
-                    if (p->factories >= (p->pop * e->colonist_oper_factories)) {
+                    if (!cond_build_ind) {
                         p->slider[PLANET_SLIDER_TECH] += p->slider[PLANET_SLIDER_ECO];
                     } else {
                         p->slider[PLANET_SLIDER_IND] += p->slider[PLANET_SLIDER_ECO];
@@ -1484,9 +1493,6 @@ static void game_turn_finished_slider(struct game_s *g)
                 }
             }
         }
-        if (BOOLVEC_IS1(p->finished, FINISHED_TERRAF)) {
-            game_add_planet_to_build_finished(g, pli, pi, FINISHED_TERRAF);
-        }
         if (BOOLVEC_IS1(p->finished, FINISHED_SHIELD)) {
             p->slider[PLANET_SLIDER_TECH] += p->slider[PLANET_SLIDER_DEF];
             p->slider[PLANET_SLIDER_DEF] = 0;
@@ -1567,11 +1573,9 @@ struct game_end_s game_turn_process(struct game_s *g)
         BOOLVEC_TBL_COPY1(old_contact, g->eto[i].contact, i, PLAYER_NUM);
         old_focus[i] = g->planet_focus_i[i];
     }
-#if 0
     if ((g->year > 40) && (!rnd_0_nm1(30, &g->seed)) && (copyprot_status == 0)) {
         copyprot_status = 1;
     }
-#endif
     game_turn_countdown_ceasefire(g);
     game_turn_update_mood_blunder(g);
     game_update_have_reserve_fuel(g);
@@ -1587,10 +1591,9 @@ struct game_end_s game_turn_process(struct game_s *g)
     game_update_have_reserve_fuel(g);
     game_turn_update_trade(g);
     game_spy_build(g);
-    game_update_production(g);
-    game_debug_dump_sliders(g, false);
-    game_debug_dump_race_techs(g, false);
-    game_debug_dump_race_spending(g, false);
+    if (!game_num_hidden_child_labor_fix) {
+        game_update_production(g);
+    }
     game_turn_diplo_adjust(g);
     game_turn_build_def(g);
     game_turn_build_ship(g);
@@ -1646,6 +1649,9 @@ struct game_end_s game_turn_process(struct game_s *g)
     if (game_event_run(g, &game_end)) {
         return game_end;
     }
+    if (game_turn_check_end(g, &game_end)) {
+        return game_end;
+    }
     for (int pli = 0; pli < g->galaxy_stars; ++pli) {
         player_id_t looter;
         looter = g->planet[pli].artifact_looter;
@@ -1673,6 +1679,7 @@ struct game_end_s game_turn_process(struct game_s *g)
         }
     }
     if (1
+      && (!g->gaux->flag_cheat_elections)
       && (game_num_council_years != 0)
       && (((g->year % game_num_council_years) == 0) || (!g->election_held))
       && (num_alive > 2)
@@ -1698,11 +1705,9 @@ struct game_end_s game_turn_process(struct game_s *g)
     }
     game_fleet_unrefuel(g);
     game_update_production(g);
-#if 0
     if (copyprot_status == 1) {
-        copyprotection_check();
+        ui_copyprotection_check(g);
     }
-#endif
     for (player_id_t i = PLAYER_0; i < g->players; ++i) {
         game_update_eco_on_waste(g, i, false);
     }
@@ -1712,11 +1717,11 @@ struct game_end_s game_turn_process(struct game_s *g)
         g->planet_focus_i[i] = old_focus[i]; /* FIXME should not be needed anymore */
     }
     ++g->year;
-#if 0
     if (copyprot_status == 1) {
-        copyprotection_lose(&game_end);
+        ui_copyprotection_lose(g, &game_end);
         return game_end;
     }
+#if 0
     game_turn_show_newships(g); /* handled in game_turn_start_messages */
 #endif
     /* MOO1 autosaves here every 5 turns */
@@ -1730,17 +1735,14 @@ struct game_end_s game_turn_process(struct game_s *g)
     game_turn_finished_slider(g);
     game_turn_claim(g);
     game_update_within_range(g);
+    if (game_num_extended_reloc_range) {
+        game_update_reloc_dest(g);
+    }
     game_update_visibility(g);
     if (g->gaux->local_players == 1) {
         game_turn_update_final_war(g);  /* MOO1 does this here, after update_tech_util etc */
     }
     game_remove_empty_fleets(g);
     game_planet_update_home(g);
-    for (int i = 0; i < g->galaxy_stars; ++i) {
-        planet_t *p = &(g->planet[i]);
-        if (p->owner != PLAYER_NONE && BOOLVEC_IS1(p->extras, PLANET_EXTRAS_GOVERNOR)) {
-            game_planet_govern(g, p);
-        }
-    }
     return game_end;
 }
