@@ -46,7 +46,6 @@ struct ui_battle_data_s {
     uint8_t frame_missile;
     bool flag_scanning;
     battle_side_i_t scan_side;
-    struct draw_stars_s stars;
     int16_t oi_ai;
     int16_t oi_missile;
     int16_t oi_special;
@@ -56,6 +55,7 @@ struct ui_battle_data_s {
     int16_t oi_scan;
     int16_t oi_auto;
     int16_t oi_done;
+    int16_t oi_grid;
     int16_t oi_area[BATTLE_AREA_H][BATTLE_AREA_W];
     ui_cursor_area_t cursor[1 + BATTLE_AREA_W * BATTLE_AREA_H];
 };
@@ -77,7 +77,7 @@ static void ui_battle_draw_scan_cb(void *vptr)
         y = i * 32 + 5;
         lbxgfx_draw_frame(46, y - 1, ui_data.gfx.space.vs2, UI_SCREEN_W, ui_scale);
         ui_draw_filled_rect(6, y, 38, y + 29, 0, ui_scale);
-        ui_draw_stars(6, y + 1, 0, 32, &d->stars, ui_scale);
+        ui_draw_stars(6, y + 1, 0, 32, ui_scale);
         ui_battle_draw_item(bt, itembase + i, 6, y + 1);
         lbxfont_select(2, 0xd, 0, 0);
         lbxfont_print_str_normal(52, y + 3, b->name, UI_SCREEN_W, ui_scale);
@@ -132,7 +132,7 @@ static void ui_battle_draw_scan_cb(void *vptr)
     }
     uiobj_finish_frame();
     ui_draw_copy_buf();
-    ui_draw_set_stars_xoffs(&d->stars, d->scan_side == SIDE_R);
+    ui_draw_set_stars_xoffs(d->scan_side == SIDE_R);
     d->frame_ship = (d->frame_ship + 1) % 5;
 }
 
@@ -172,11 +172,11 @@ static void ui_battle_draw_focusinfo(const struct battle_s *bt)
         if (b->num > 0) {
             char buf[32];
             int h1 = b->hp1, h2 = b->hp2, ha = h1 - b->hploss, pos;
-            pos = sprintf(buf, "%i/", ha);
+            pos = lib_sprintf(buf, sizeof(buf), "%i/", ha);
             if (h1 < h2) {
                 buf[pos++] = 2;
             }
-            sprintf(&buf[pos], "%i", h1);
+            lib_sprintf(&buf[pos], sizeof(buf) - pos, "%i", h1);
             lbxfont_print_str_right(92, 194, buf, UI_SCREEN_W, ui_scale);
         }
     }
@@ -192,6 +192,7 @@ static void ui_battle_clear_ois(struct ui_battle_data_s *d)
     d->oi_scan = UIOBJI_INVALID;
     d->oi_auto = UIOBJI_INVALID;
     d->oi_done = UIOBJI_INVALID;
+    d->oi_grid = UIOBJI_INVALID;
     for (int y = 0; y < BATTLE_AREA_H; ++y) {
         for (int x = 0; x < BATTLE_AREA_W; ++x) {
             d->oi_area[y][x] = UIOBJI_INVALID; /* BUG? not done in MOO1 */
@@ -330,6 +331,7 @@ static void ui_battle_draw_bottom_add_ois(const struct battle_s *bt)
         d->oi_retreat = uiobj_add_t0(241, 193, "", ui_data.gfx.space.retreat, MOO_KEY_r);
     }
     d->oi_done = uiobj_add_t0(297, 193, "", ui_data.gfx.space.done, MOO_KEY_d);
+    d->oi_grid = uiobj_add_inputkey(MOO_KEY_g);
     d->oi_wait = uiobj_add_t0(274, 193, "", ui_data.gfx.space.wait, MOO_KEY_w);
 #if 0
     d->oi_auto = uiobj_add_t0(99, 193, "", ui_data.gfx.space.autob, MOO_KEY_a);
@@ -903,7 +905,7 @@ void ui_battle_draw_planetinfo(const struct battle_s *bt, bool side_r)
             if ((!bt->s[b->side].flag_base_missile) && (w->nummiss == 1)) {
                 w = &(tbl_shiptech_weap[b->wpn[1].t]);
             }
-            sprintf(buf, "3 %s %s", *w->nameptr, game_str_bt_launch);
+            lib_sprintf(buf, sizeof(buf), "3 %s %s", *w->nameptr, game_str_bt_launch);
             lbxfont_print_str_normal(x + 10, y + 71, buf, UI_SCREEN_W, ui_scale);
         }
         lbxfont_select(2, 0xb, 0, 0);
@@ -1040,6 +1042,14 @@ void ui_battle_draw_arena(const struct battle_s *bt, int itemi, int dmode)
     lbxgfx_draw_frame(0, 0, d->gfx_bg, UI_SCREEN_W, ui_scale);
     lbxfont_set_temp_color(0);
     lbxfont_select_set_12_4(2, 0xd, 0, 0);
+    if (ui_extra_enabled && ui_data.battle.show_grid) {
+        for (int x = 0; x < 10; ++x) {
+            ui_draw_line1(x * 32, 0, x * 32, 24 * 8 - 1, 4, ui_scale);
+        }
+        for (int y = 0; y < 8; ++y) {
+            ui_draw_line1(0, y * 24, 32 * 10 - 1, y * 24, 4, ui_scale);
+        }
+    }
     for (int i = 0; i <= bt->items_num; ++i) {
         if ((i != itemi) || (dmode == 0/*normal*/)) {
             const struct battle_item_s *b;
@@ -1158,7 +1168,7 @@ void ui_battle_draw_damage(const struct battle_s *bt, int target_i, int target_x
     int target_x_hit, target_y_hit, si, v4, ax, ay, scale;
     bool flag_quick;
     si = (b->hp1 > 0) ? b->hp1 * 3 : 1;
-    if ((b->num > 0) || (si < damage)) {
+    if ((b->num == 0) || (si < damage)) {
         v4 = si;
     } else {
         v4 = damage; /* & 0xffff */
@@ -1928,6 +1938,9 @@ ui_battle_action_t ui_battle_turn(const struct battle_s *bt)
         }
         return UI_BATTLE_ACT_DONE;
     }
+    if (oi == d->oi_grid) {
+        ui_data.battle.show_grid = !ui_data.battle.show_grid;
+    }
     /*4eddd*/
     if (oi == d->oi_wait) {
         ui_sound_play_sfx_24();
