@@ -504,10 +504,14 @@ int ui_late_init(void)
       || init_lbx_ships()
       || set_ui_icon()
       || hw_video_init(UI_SCREEN_W, UI_SCREEN_H)
-      || lbxpal_init()
     ) {
         return 1;
     }
+    ui_palette_clear();
+    hw_video_refresh_palette();
+    if (lbxpal_init()) {
+        return 1;
+    }
     if (opt_audio_enabled) {
         const uint8_t sounds[] = { /* sounds used by ui or game code */
             0x24, 0x06, /* these are the most common sounds and are prepared first */
