@@ -46,9 +46,7 @@ struct fleet_data_s {
     int pos;
     int num;
     int lines;
-    int frame;
     int order_i;
-    struct draw_stars_s s;
     uint8_t *gfx_fleetbrb;
 };
 
@@ -130,19 +128,22 @@ static void fleet_draw_cb(void *vptr)
                 x0 = 44 * j + 48;
                 ui_draw_filled_rect(x0, y0, x0 + 36, y0 + 25, 0, ui_scale);
                 if (!FLEET_IS_ENROUTE(fi)) {
-                    struct draw_stars_s temps;
-                    temps.xoff1 = 0;
-                    temps.xoff2 = 0;
-                    ui_draw_stars(x0, y0 + 1, j * 10, 37, &temps, ui_scale);
+                    int tmp_xoff1 = ui_data.starmap.stars_xoff1;
+                    int tmp_xoff2 = ui_data.starmap.stars_xoff2;
+                    ui_data.starmap.stars_xoff1 = 0;
+                    ui_data.starmap.stars_xoff2 = 0;
+                    ui_draw_stars(x0, y0 + 1, j * 10, 37, ui_scale);
+                    ui_data.starmap.stars_xoff1 = tmp_xoff1;
+                    ui_data.starmap.stars_xoff2 = tmp_xoff2;
                 } else {
-                    ui_draw_stars(x0, y0 + 1, j * 5, 37, &d->s, ui_scale);
+                    ui_draw_stars(x0, y0 + 1, j * 5, 37, ui_scale);
                 }
                 gfx_ship = ui_data.gfx.ships[sd[j].look];
                 lbxgfx_set_frame_0(gfx_ship);
                 if (!FLEET_IS_ENROUTE(fi)) {
                     lbxgfx_draw_frame(x0, y0 + 1, gfx_ship, UI_SCREEN_W, ui_scale);
                 } else {
-                    for (int f = 0; f <= d->frame; ++f) {
+                    for (int f = 0; f <= ui_data.starmap.frame_ship; ++f) {
                         lbxgfx_draw_frame(x0, y0 + 1, gfx_ship, UI_SCREEN_W, ui_scale);
                     }
                 }
@@ -154,8 +155,8 @@ static void fleet_draw_cb(void *vptr)
     for (int i = num; i < FLEET_LINES; ++i) {
         ui_draw_filled_rect(7, i * 33 + 17, 40, i * 33 + 42, 0x3a, ui_scale);
     }
-    d->frame = (d->frame + 1) % 5;
-    ui_draw_set_stars_xoffs(&d->s, false);
+    ui_data.starmap.frame_ship = (ui_data.starmap.frame_ship + 1) % 5;
+    ui_draw_set_stars_xoffs(false);
     lbxfont_select(2, 6, 0, 0);
     lbxfont_print_num_right(137, 185, e->ship_maint_bc, UI_SCREEN_W, ui_scale);
 }
@@ -326,6 +327,12 @@ static sort_cb_t * const sort_cb_tbl[UI_SORT_NUM * 2] = {
     fleet_sort_inc_ship5
 };
 
+static void ui_fleet_sort(struct fleet_data_s *d)
+{
+    qsort(ui_data.sorted.index, d->num, sizeof(ui_data.sorted.index[0]), sort_cb_tbl[d->order_i]);
+    ui_data.sorted.fleet_order[d->api] = d->order_i;
+}
+
 /* -------------------------------------------------------------------------- */
 
 int ui_fleet(struct game_s *g, player_id_t active_player)
@@ -347,8 +354,12 @@ int ui_fleet(struct game_s *g, player_id_t active_player)
     d.order_i = 0;
     game_update_maint_costs(g);
     ui_fleet_sub(&d);
-    d.frame = 0;
+    ui_data.starmap.frame_ship = 0;
     d.lines = 0;
+    if (ui_extra_enabled) {
+        d.order_i = ui_data.sorted.fleet_order[active_player];
+        ui_fleet_sort(&d);
+    }
 
     uiobj_table_clear();
 
@@ -436,7 +447,7 @@ int ui_fleet(struct game_s *g, player_id_t active_player)
                     ++v;
                 }
                 d.order_i = v;
-                qsort(ui_data.sorted.index, d.num, sizeof(ui_data.sorted.index[0]), sort_cb_tbl[d.order_i]);
+                ui_fleet_sort(&d);
             }
         }
         if (!flag_done) {
@@ -452,7 +463,7 @@ int ui_fleet(struct game_s *g, player_id_t active_player)
             UIOBJ_CLEAR_LOCAL();
 
             oi_ok = uiobj_add_t0(268, 181, "", ui_data.gfx.starmap.fleetbut_ok, MOO_KEY_SPACE);
-            oi_view = uiobj_add_t0(180, 181, "", ui_data.gfx.starmap.fleetbut_view, MOO_KEY_v);
+            oi_view = uiobj_add_t0(180, 181, "", ui_data.gfx.starmap.fleetbut_view, (ui_illogical_hotkey_fix ? MOO_KEY_s : MOO_KEY_v));
             if (d.pos > 0) {
                 oi_up = uiobj_add_t0(307, 24, "", ui_data.gfx.starmap.fleetbut_up, MOO_KEY_COMMA);
             }
@@ -460,7 +471,7 @@ int ui_fleet(struct game_s *g, player_id_t active_player)
                 oi_down = uiobj_add_t0(307, 156, "", ui_data.gfx.starmap.fleetbut_down, MOO_KEY_PERIOD);
             }
             if ((g->eto[active_player].shipdesigns_num > 1) && !flag_scrap) {
-                oi_scrap = uiobj_add_t0(224, 181, "", ui_data.gfx.starmap.fleetbut_scrap, MOO_KEY_s);
+                oi_scrap = uiobj_add_t0(224, 181, "", ui_data.gfx.starmap.fleetbut_scrap, (ui_illogical_hotkey_fix ? MOO_KEY_c : MOO_KEY_s));
             } else {
                 lbxgfx_set_new_frame(ui_data.gfx.starmap.fleetbut_scrap, 1);
                 lbxgfx_draw_frame(224, 181, ui_data.gfx.starmap.fleetbut_scrap, UI_SCREEN_W, ui_scale);
