@@ -448,15 +448,7 @@ do_accept:
             ui_starmap_orbit_own_draw_cb(&d);
             uiobj_table_clear();
             UIOBJ_CLEAR_LOCAL();
-            oi_f2 = uiobj_add_inputkey(MOO_KEY_F2);
-            oi_f3 = uiobj_add_inputkey(MOO_KEY_F3);
-            oi_f4 = uiobj_add_inputkey(MOO_KEY_F4);
-            oi_f5 = uiobj_add_inputkey(MOO_KEY_F5);
-            oi_f6 = uiobj_add_inputkey(MOO_KEY_F6);
-            oi_f7 = uiobj_add_inputkey(MOO_KEY_F7);
-            oi_f8 = uiobj_add_inputkey(MOO_KEY_F8);
-            oi_f9 = uiobj_add_inputkey(MOO_KEY_F9);
-            oi_f10 = uiobj_add_inputkey(MOO_KEY_F10);
+            STARMAP_UIOBJ_FILL_FX();
             if (!ui_sm_explicit_cursor_context || kbd_is_modifier(MOO_MOD_ALT)) {
                 ui_starmap_fill_oi_tbls(&d);
                 ui_cursor_setup_area(2, &ui_cursor_area_tbl[3]);
